/**
 * MPQAudioWorkerRecThread.java: Recording worker thread class.
 *            Copyright (C) 2012,2013 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2012-12-04 donfede@casagrau.org
 * credit to: Reto Meier, EMEA Android Developer Advocate, Google UK; for a record and playback example -- http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
 *			  Stjepan Rajko, urbanSTEW -- (GPL) RehearsalAssistant/RehearsalAudioRecorder.java
 *			  cristi (Cristian Baita) -- http://indyvision.net/2010/02/android-threads-tutorial-part-3/
 * $Id: MPQAudioWorkerRecThread.java,v 1.4 2013-01-29 03:24:43 donfede Exp $
 */

package org.casagrau.mpq.audio_service;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.Context;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;



public class MPQAudioWorkerRecThread extends MPQAudioWorkerThread {
	private static final String LOGTAG = "MPQAudioWorkerRecThread";
	
	/** 
	 * Constructor for the worker thread.
	 * {@link #MPQAudioWorkerThread}  
	 */
	public MPQAudioWorkerRecThread(Handler parentHandler, String filename, long filepos, int state, Context context) {
		super(parentHandler, filename, filepos, state, context);
	}
		
	
	/**
	 * Record audio to a wave file.
	 * This code is meant to be started from a thread, and will run until 
	 * a max time limit is hit (recording or pause), at which point the file will be closed.
	 * Credit to: Reto Meier, EMEA Android Developer Advocate, Google UK -- http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
	 * @param intent with the action and extra content to process. 
	 * 					Extra ARG_FILENAME is checked for to open.
	 * @throws IllegalStateException if there are errors writing the file
	 * @throws IOException if there are errors opening the audio device
	 */
	//FIXME: make private
	public void run() { 
		// FIXME: get these values from preferences
		int lfrequency = 11025; 
		int lchannelConfiguration = AudioFormat.CHANNEL_IN_MONO;
		int laudioEncoding = AudioFormat.ENCODING_PCM_16BIT; 
	    Log.d(LOGTAG, "runREC() 00 freq: " + lfrequency + " lchan: " + lchannelConfiguration + " lenc: " + laudioEncoding);

    	short lchannelCount;
    	if (AudioFormat.CHANNEL_IN_MONO == lchannelConfiguration) lchannelCount = 1;
    	else if (AudioFormat.CHANNEL_IN_STEREO == lchannelConfiguration) lchannelCount = 2;
    	else throw new IllegalStateException("unsuported channel configuration: " + lchannelConfiguration);

    	short lsampleBit = 0; // sample bitrate
    	if (AudioFormat.ENCODING_PCM_16BIT == laudioEncoding) lsampleBit = 16;
    	else if (AudioFormat.ENCODING_PCM_8BIT == laudioEncoding) lsampleBit = 8;
    	else throw new IllegalStateException("unsuported sample bitrate: " + lsampleBit);
    	
    	int lframeSize = (lsampleBit/8)*lchannelCount;
		Log.d(LOGTAG, "runREC() 06  mStateThread: " + mStateThread);
		
		File lFile; // Local file handle;

	    // Confirm parent directory exists.
		lFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + AUDIO_BASEDIR);
	    Log.d(LOGTAG, "runREC() 10 file basedir: " + lFile);

	    if (!lFile.isDirectory()) {
		    Log.d(LOGTAG, "runREC() 12 DIR not directory - del");
	    	lFile.delete();
	    }
	    if (!lFile.exists() ) {
		    Log.d(LOGTAG, "runREC() 14 DIR no exists - mkdir");
	    	lFile.mkdirs();
	    }
	    	
		lFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + mFilename);
	    Log.d(LOGTAG, "runREC() 20 FILE file: " + lFile);
		// Delete any previous recording.
		if (lFile.exists())
			lFile.delete();

		// Create the new file.
		try {
			lFile.createNewFile();
		} catch (IOException e) {
			throw new IllegalStateException("Failed to create " + lFile.toString());
		}

		try {
			// Create a new AudioRecord object to record the audio.
			int lbufferSize = AudioRecord.getMinBufferSize(lfrequency, lchannelConfiguration, laudioEncoding);
			lbufferSize = 2 * lbufferSize;
			byte[] lbuffer = new byte[lbufferSize];
	        Log.d(LOGTAG, "runREC() 40  bufSize(2xmin): " + lbufferSize);
			AudioRecord laudioRecord = new AudioRecord(MediaRecorder.AudioSource.MIC,
					lfrequency, lchannelConfiguration,
					laudioEncoding, lbufferSize);

			if (laudioRecord.getState() != AudioRecord.STATE_INITIALIZED)
				throw new IOException("runREC() AudioRecord initialization failed");
//			audioRecord.setRecordPositionUpdateListener(updateListener);
//			audioRecord.setPositionNotificationPeriod(framePeriod);
			

			// Write initial WAVE file headers. -- credit: Stjepan Rajko
			mFileHandle = new RandomAccessFile(lFile, "rw");
	    	
			mFileHandle.setLength(0); // Set file length to 0, to prevent unexpected behavior in case the file already existed
			mFileHandle.writeBytes("RIFF");
			mFileHandle.writeInt(0); // Final file size not known yet, write 0 
			mFileHandle.writeBytes("WAVE");
			mFileHandle.writeBytes("fmt ");
			mFileHandle.writeInt(Integer.reverseBytes(16)); // Sub-chunk size, 16 for PCM
			mFileHandle.writeShort(Short.reverseBytes((short) 1)); // AudioFormat, 1 for PCM
			mFileHandle.writeShort(Short.reverseBytes(lchannelCount));// Number of channels, 1 for mono, 2 for stereo
			mFileHandle.writeInt(Integer.reverseBytes(lfrequency)); // Sample rate
			mFileHandle.writeInt(Integer.reverseBytes(lfrequency*lsampleBit*lchannelCount/8)); // Byte rate, SampleRate*NumberOfChannels*BitsPerSample/8
			mFileHandle.writeShort(Short.reverseBytes((short)(lchannelCount*lsampleBit/8))); // Block align, NumberOfChannels*BitsPerSample/8
			mFileHandle.writeShort(Short.reverseBytes(lsampleBit)); // Bits per sample
			mFileHandle.writeBytes("data");
			mFileHandle.writeInt(0); // Data chunk size not known yet, write 0
	//FIXME: review, cleanup - buffer from wave example -- don't expect it's useful here...			buffer = new byte[framePeriod*lsampleBit/8*lchannelCount];
////			short[] buffer = new short[bufferSize];
			
			// Record for up to $X seconds FIXME: make this a Preferences setting
			laudioRecord.startRecording();
			long lnowTime = System.currentTimeMillis();
			SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			String lstartTimeStr = formatter.format(new Date(lnowTime));
			Log.d(LOGTAG, "runREC() 50  str start: " + lstartTimeStr);

			long lpayloadSize = 0;// Size of the audio recorded, used later in wave headers.
			mFilepos = 0;
			sendPMsgPos(mFilepos, false);
			int lbufferReadSize;	// Variable to store the size read into the buffer.
			// Recording loop, while state is to record, and we have not hit max time limit
			while ( (MPQAudioService.FLG_SRV_STATE_REC == mStateThread) || 
					(MPQAudioService.FLG_SRV_STATE_PAUSE_REC == mStateThread) ) {
				lbufferReadSize = laudioRecord.read(lbuffer, 0, lbufferSize);
				
				if (MPQAudioService.FLG_SRV_STATE_REC == mStateThread) {
	    			lpayloadSize = lpayloadSize + lbufferReadSize;
	    			mFileHandle.write(lbuffer, 0, lbufferReadSize);
	    			mFilepos = cursorFrameToMS(lpayloadSize, lframeSize, lfrequency);
	    			sendPMsgPos(mFilepos, false);
	        		Log.d(LOGTAG, "runREC() 72      IN while loop recording buffer.len: " + lbuffer.length + " payloadSize: " + lpayloadSize );
	        		Log.d(LOGTAG, "runREC() 74               loop recording filepos: " +  + mFileHandle.getFilePointer() );
				} // else discard recorded buffer 

				lnowTime = System.currentTimeMillis();
	    		Log.d(LOGTAG, "runREC() 78      IN while loop mState: " + mStateThread + " bufsize: " + lbuffer.length + " nowTime:" + formatter.format(new Date(lnowTime)) );
	    		if (MPQAudioService.FLG_SRV_STATE_PAUSE_REC == mStateThread) {
	    			try {
	    				Thread.sleep(AUDIO_PAUSE_MS);
	    			} catch (Exception e) {
	    				e.getLocalizedMessage();
	    			}
	        		Log.d(LOGTAG, "runREC() 79      IN while PAUSE WAIT mState: " + mStateThread + " nowTime:" + formatter.format(new Date(lnowTime)) );
				} // END if pause wait
			} // END while recording loop
			Log.d(LOGTAG, "runREC() 80  OUT of while loop to record now time:" + formatter.format(new Date(lnowTime)) );
			laudioRecord.stop();
			laudioRecord.release();

			mFileHandle.getFD().sync();
			// Cleanup and update WAV headers with size
			try	{
				mFileHandle.seek(4); // Write size to RIFF header
				mFileHandle.writeInt(Integer.reverseBytes(40+(int)lpayloadSize)); // FIXME: *** evil hack - casting payloadSize to int -- need to review upper limit (set by WAVE format) and adapt accordingly ...
				mFileHandle.seek(40); // Write size to Subchunk2Size field
				mFileHandle.writeInt(Integer.reverseBytes((int)lpayloadSize)); // FIXME: *** evil hack - casting payloadSize to int -- need to review upper limit (set by WAVE format) and adapt accordingly ...
				mFileHandle.close();
			} catch(IOException e) {
				Log.e(LOGTAG, "I/O exception occured while closing output file");
	//FIXME: likely need to implement an error state...				state = State.ERROR;
				e.getLocalizedMessage();
			}
			Log.d(LOGTAG, "runREC() 90  WAVE headers updated - payloadSize: " + lpayloadSize + " mStateThread: " + mStateThread);
			double lfrequencyMS = lfrequency / 1000.0; // cycles per millisecond
	    	long lframeCount = lpayloadSize / lframeSize;    	
	    	long lfileMS = (long) (lframeCount / lfrequencyMS); // FIXME: review/re-think if this float division is necessary...
	    	Log.d(LOGTAG, "runREC() 24  fileMS: " + lfileMS + " frameCount: " + lframeCount + " lfreq: " + lfrequency + " lfreqMS: " + lfrequencyMS);
//////////////			sendPMsgLen(lfileMS);

			// Set state to stopped, and send updates.
			mFilepos = 0;
			sendPMsgPos(mFilepos, true);
			if (MPQAudioService.FLG_SRV_STATE_STOP != mStateThread) {
	    		Log.d(LOGTAG, "runREC() 94 send startService intent STOP");
	    		sendSvcIntentStop();
			}
    		Log.d(LOGTAG, "runREC() 97 send startService intent OPEN");
    		sendSvcIntentOpen();
    		
    		Log.d(LOGTAG, "runREC() 98 send startService intent AA to analyze audio");
    		sendSvcIntentAA();

		} catch (Exception e) {
			Log.e(LOGTAG,"runREC() Recording Failed e:" + e.getLocalizedMessage());
		}
		Log.d(LOGTAG, "runREC() 99  ");
	} // END runREC()


    
} // END class MPQAudioWorkerRecThread
