/**
 * SessionsListFragment.java: List of sessions.
 *            Copyright (C) 2012 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2012-09-23 donfede@casagrau.org
 * credit to: package com.example.android.notepad/NotesList.java
 *
 * $Id: SessionsListFragment.java,v 1.20 2013-02-09 20:07:45 donfede Exp $
 */
package org.casagrau.mpq.mpq_android;

import org.casagrau.mpq.mpq_android.R.layout;
import org.casagrau.mpq.mpq_android.SessionsDBProvider.SessionsDB;

import android.content.ContentUris;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.support.v4.widget.SimpleCursorAdapter;
import android.support.v4.widget.SimpleCursorAdapter.ViewBinder;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;

/**
 * Fragment with GUI for Sessions List.
 */
public class SessionsListFragment extends Fragment implements
		ListView.OnItemClickListener, LoaderManager.LoaderCallbacks<Cursor> {
	private static final String LOGTAG = "SessionsListFragment";
	
	// Unique integer identifiers for Loaders; FIXME: rather warty (repeated in DetailFragment)... :(
	private static final int LDR_SESSIONS = 2;

	// Adapters to bind to CursorLoaders
	SimpleCursorAdapter mAdapterSessions;

	// Parameterless constructor, required by framework
	public SessionsListFragment() {
		super();
	}
		
	/**
	 * Create the view for this fragment - bringing XML layout with list that has customized columns.
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, 
			Bundle savedInstanceState) {	
		View v = inflater.inflate(R.layout.fragment_sessionslist, container, false);
		return v;
	}
	
	
	@SuppressWarnings("deprecation") // benign complaints about a temporary null cursor used...
	@Override
	public void onResume() {
		super.onResume();

		// bind adapter to ListView and connect listener
		String[] fromColumns = {SessionsDB.PRACTICE_DATE_STAMP, 
				SessionsDB.PRACTICE_LENGTH_TOTAL, 
				SessionsDB.PRACTICE_LENGTH_NOISE };
		int[] toViews = {R.id.list_date_stamp, 
				R.id.list_length_total, 
				R.id.list_length_noise};
		mAdapterSessions = new SimpleCursorAdapter(getActivity(),
				layout.fragment_sessionslist_item, null,
				fromColumns, toViews );

		// Customize the output of the list layout.  For now, convert UNIX time seconds into human readable time.
		mAdapterSessions.setViewBinder(new ViewBinder() {
			public boolean setViewValue(View aView, Cursor aCursor,
					int col) {
				Log.d(LOGTAG, "mAdapterSessions.setViewBinder() 00 col: " + col);
				long lMSec;
				String lStr;
				// FIXME: a lot of cleanup/tidy that could be done in this if() statement ... for now - press on.
				if (1 == col) { 
					lMSec = aCursor.getLong(col);
			        lStr = SessionsDB.longDateString(lMSec);
					TextView textView = (TextView) aView;
					textView.setText(lStr);
					return true;
				} else if (2 == col) {
					lMSec = aCursor.getLong(col);
					lStr = "Len: " + SessionsDB.longSessionMSecString(lMSec);
					TextView textView = (TextView) aView;
					textView.setText(lStr);
					return true;
				} else if (3 == col) {
					lMSec = aCursor.getLong(col);
					lStr =  "Noise: " + SessionsDB.longSessionMSecString(lMSec);
					TextView textView = (TextView) aView;
					textView.setText(lStr);
					return true;
				}
				return false;
			}
		}); // END setViewBinder() 

		// Prepare the Loaders, triggering callback methods.
		getLoaderManager().initLoader(LDR_SESSIONS, null, this);
		
		ListView listView = (ListView) getActivity().findViewById(R.id.listview);
		listView.setAdapter(mAdapterSessions);
		
		listView.setOnItemClickListener(this); 
        listView.setOnCreateContextMenuListener(this);
	} // END onResume()

	public void onItemClick(AdapterView<?> l, View v, int position, long rowId) {
		Log.d(LOGTAG, "onItemClick() pos: " + position + ":rowId:" + rowId);

        MPQMainTabActivity lmActivity = (MPQMainTabActivity) getActivity(); 
		lmActivity.updateTab(getString(R.string.frag_tab2_detail), rowId);
	}


	//////////
	// Context Menu creation listener and callback handler - credit to package com.example.android.notepad;
	/**
	 * Create a context menu, to Add, Edit, or Delete a practice session from the list.
	 * credit to: package com.example.android.notepad/NotesList.java
	 */
    @Override
    public void onCreateContextMenu(ContextMenu menu, View view, ContextMenuInfo menuInfo) {
        AdapterView.AdapterContextMenuInfo info;
        try {
             info = (AdapterView.AdapterContextMenuInfo) menuInfo;
        } catch (ClassCastException e) {
            Log.e(LOGTAG, "onCreateContextMenu() unable to access menuInfo", e);
            return;
        }

        Cursor cursor = (Cursor) mAdapterSessions.getItem(info.position);
        if (cursor == null) {
            return; // For some reason the requested item isn't available, do nothing
        }

        // Inflate menu from XML resource
        MenuInflater inflater = getActivity().getMenuInflater();
        inflater.inflate(R.menu.sessionslist_context_menu, menu);
        
        // Set the context menu header to the formatted session date stamp
        long ldateSec = cursor.getLong(1); // FIXME: ugly hard-coded constant, pointing to the column in the cursor we want... this should be abstracted...
        String ldateStr = SessionsDB.longDateString(ldateSec);
        menu.setHeaderTitle(ldateStr);         
    } // END onCreateContextMenu()
       
	/**
	 * Respond to context menu input from the user.
	 * credit to: package com.example.android.notepad/NotesList.java
	 */
    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterView.AdapterContextMenuInfo info;
        try {
             info = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
        } catch (ClassCastException e) {
            Log.e(LOGTAG, "onContextItemSelected() unable to access menuInfo", e);
            return false;
        }

        Log.d(LOGTAG, "onContextItemSelected() 00 item: " + item);  
        Log.d(LOGTAG, "onContextItemSelected() 02   info: " + info + " i.id: " + info.id);        
        Uri lsessionUri = ContentUris.withAppendedId(SessionsDB.CONTENT_PRACTICES_URI, info.id);
        
        Log.d(LOGTAG, "onContextItemSelected() 04   uri: " + lsessionUri);
		
        MPQMainTabActivity lmActivity = (MPQMainTabActivity) getActivity(); // Local handle to the parent activity.

        switch (item.getItemId()) {
        case R.id.slcontext_add:
        	Log.d(LOGTAG, "onContextItemSelected() 12 ADD  ");
    		lmActivity.updateTab(getString(R.string.frag_tab2_detail),
    				SessionDetailFragment.ARG_DETAIL_FLAG_START);
            return true;
        case R.id.slcontext_open:
        	Log.d(LOGTAG, "onContextItemSelected() 13 OPEN ");
    		lmActivity.updateTab(getString(R.string.frag_tab2_detail),
    				info.id);
            return true;
        case R.id.slcontext_delete:
            // Delete the note that the context menu is for
        	Log.d(LOGTAG, "onContextItemSelected() 14   DELETE");
Log.e(LOGTAG, "onContextItemSelected() 14a   FIXME: DELETE - make sure to try and delete file first; maybe send call to transport control");
        	getActivity().getContentResolver().delete(lsessionUri, null, null);
        	Log.d(LOGTAG, "onContextItemSelected() 15   delete finished");
            return true;
        default:
        	Log.d(LOGTAG, "onContextItemSelected() 18   ");
        	return super.onContextItemSelected(item);
        }
    } // END onContextItemSelected()

	
	
	// ///////////////////////////////////////////////////
	// LoaderManager.LoaderCallbacks<Cursor> implementations -- FIXME: another very warty area - with a lot of code repeated from the DetailFragment...
	/**
	 * LoaderManager callback to create new loaders.
	 */
	public Loader<Cursor> onCreateLoader(int id, Bundle args) {
		Log.d(LOGTAG, "onCreateLoader() 00 id: " + id);

		String[] lproj;
		String lselc = null;
		String[] lsela = { "" };
		Uri luri;
		
		// Setup projection		
		Log.d(LOGTAG, "onCreateLoader() 14 id: " + id);			
		lproj = new String[] { SessionsDB._ID, 
				SessionsDB.PRACTICE_DATE_STAMP, 
				SessionsDB.PRACTICE_LENGTH_TOTAL,
				SessionsDB.PRACTICE_LENGTH_NOISE};
		luri = SessionsDB.CONTENT_PRACTICES_URI;

		Log.d(LOGTAG, "onCreateLoader() 20 id: " + id);		
		// Get cursor
		CursorLoader lcursLoader = new CursorLoader(getActivity(),
				luri, lproj, lselc, lsela, null);
		Log.d(LOGTAG, "onCreateLoader() 99");
		return lcursLoader;
	} // END onCreateLoader()

	/**
	 * LoaderManager callback to process when loading cursor data is completed.
	 */
	public void onLoadFinished(Loader<Cursor> loader, Cursor data) {
		Log.d(LOGTAG, "onLoadFinished() 00");
		mAdapterSessions.swapCursor(data);
		Log.d(LOGTAG, "onLoadFinished() 99");
		// The list should now be shown. -- FIXME: is this "now be shown"
		// necessary???
		// if (isResumed()) {
		// setListShown(true);
		// } else {
		// setListShownNoAnimation(true);
		// }
	} // END onLoadFInished()

	/**
	 * LoaderManager callback when releasing data.
	 */
	public void onLoaderReset(Loader<Cursor> loader) {
		Log.d(LOGTAG, "onLoaderReset() 00");
		mAdapterSessions.swapCursor(null);
	}


} // END class SessionsListFragment
